// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by a MIT-style license
// that can be found in the LICENSE file.

part of omnibus;

/// Event callback typedef
typedef Future<Null> OmnibusEventCallback<T>(T message);

/// Request callback typedef
typedef Future<OmnibusResponse> OmnibusRequestCallback<T>(T message);

/// Global event handler typedef
typedef Future<Null> GlobalEventHandler(OmnibusEvent event);

/// Global request handler typedef
typedef Future<List<Future<OmnibusResponse>>> GlobalRequestHandler(
    OmnibusRequest request);

/// Handler semantics
abstract class OmnibusHandler {
  /// Type (event or request channel)
  final OmnibusMessageType type;

  /// Target omnibusId
  final String omnibusId;

  /// Funnel values
  final Map<String, dynamic> funnel;

  /// Constructor
  OmnibusHandler(this.type, this.omnibusId, this.funnel);
}

abstract class OmnibusEventHandler extends OmnibusHandler {
  /// Constructor
  OmnibusEventHandler(String omnibusId,
      [Map<String, dynamic> funnel = const {}])
      : super(OmnibusMessageType.event, omnibusId, funnel);

  /// Message processing
  Future<Null> process(OmnibusEvent data);
}

abstract class OmnibusRequestHandler extends OmnibusHandler {
  /// Constructor
  OmnibusRequestHandler(String omnibusId,
      [Map<String, dynamic> funnel = const {}])
      : super(OmnibusMessageType.request, omnibusId, funnel);

  /// Message processing
  Future<OmnibusResponse> process(OmnibusRequest data);
}
