// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by a MIT-style license
// that can be found in the LICENSE file.

part of omnibus;

/// Handler index
///
/// This class is used by the Omnibus controller to index all handlers by their
/// funnel values. The handlers are linked to this class using an index number.
class _HandlerIndex {
  /// Indices for each possible value of each possible funnel key
  Map<String, Map<dynamic, Set<int>>> _values =
      new Map<String, Map<dynamic, Set<int>>>();

  /// Indices of handlers that have a funnel value for the given funnel key
  Map<String, Set<int>> _hasValue = new Map<String, Set<int>>();

  /// All indices.
  Set<int> _indices = new Set<int>();

  /// Insert new index with given funnel.
  void insertIndex(int index, Map<String, dynamic> funnel) {
    _indices.add(index);
    funnel.forEach((String key, dynamic value) {
      // A null value is treated as not set.
      if (value != null) {
        _values.putIfAbsent(key, () => new Map<dynamic, Set<int>>());
        _values[key].putIfAbsent(value, () => new Set<int>());
        _values[key][value].add(index);
        _hasValue.putIfAbsent(key, () => new Set<int>());
        _hasValue[key].add(index);
      }
    });
  }

  /// Collect indices for the given funnel.
  Set<int> findIndices(Map<String, dynamic> funnel) {
    var indices = new Set<int>.from(_indices);

    for (final key in funnel.keys) {
      if (_hasValue.containsKey(key)) {
        var exclude = new Set<int>.from(_hasValue[key]);
        if (_values.containsKey(key) && _values[key].containsKey(funnel[key])) {
          exclude = exclude.difference(_values[key][funnel[key]]);
        }
        indices = indices.difference(exclude);
      }

      // Return if indices are already empty.
      if (indices.isEmpty) {
        return indices;
      }
    }

    return indices;
  }
}
