// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by a MIT-style license
// that can be found in the LICENSE file.

library omnibus.test.funnel_test;

import 'dart:async';

import 'package:test/test.dart';
import 'package:omnibus/omnibus.dart';

import 'messages_lib.dart' as v0;

/// Count how many messages passed through the funnel.
int msgCount = 0;

void main() {
  test('Publish events while there are no handlers', () {
    var bus = new Omnibus();

    // Event with data
    bus.publishEvent(new v0.Data({'a': 'a'}));

    // Empty event
    bus.publishEvent(new v0.Data({}));
  });

  test('Various funnels', () {
    var bus = new Omnibus();

    // Bind some handlers.
    bus.bind(new v0.AbcHandler(dummyHandler, a: 'a', b: 'b', c: 'c'));

    // Publish some events.
    bus.publishEvent(new v0.Abc(a: 'a', b: 'b', c: 'c'));
    bus.publishEvent(new v0.Abc(a: 'b', b: 'b', c: 'c'));
    bus.publishEvent(new v0.Abc(a: 'a', b: 'b', c: 'a'));
  });

  test('Funnel-less handlers', () {
    var bus = new Omnibus();

    bus.bind(new v0.MessageForAllHandler(mForAllHandler));
    bus.publishEvent(new v0.MessageForAll('This message is for everyone!'));
  });
}

Future<Null> dummyHandler(v0.Abc abc) async {
  expect(abc.a, equals('a'));
  expect(abc.b, equals('b'));
  expect(abc.c, equals('c'));

  // One messages should have passed through the funnel.
  msgCount++;
  expect(msgCount, equals(1));
}

Future<Null> mForAllHandler(v0.MessageForAll m) async {
  expect(m.message, equals('This message is for everyone!'));
}
